@echo ***************************************************************
@echo *** Aggressive Games Search Tool V3.9 by Stefan Pohl (SPCC) ***
@echo ***               (C) 2022, www.sp-cc.de                    ***
@echo ***************************************************************
@echo ***     This tool uses pgn-extract (C) David J. Barnes      ***
@echo ***************************************************************
@echo ***************************************************************
@echo Gib den Dateinamen der zu untersuchenden pgn-Datei ein (ohne die Endung .pgn !!!):
@echo off
set /p filename=
set gamebase=%filename%.pgn
@echo Dieses Tool sucht nach gegnerischen Figuren auf der Reihe 1/2/3 aus Sicht des Gegners (z.B: weisser
@echo Turm auf der 7.Reihe) im Mittelspiel (geprueft ueber Materialbilanz und von Zug 15 bis Zug 60) und 
@echo nach Opfern (also temporaerer Materialvorteil fuer die Seite, die die Partie am Ende verliert).
@echo Gebe 1 ein (oder nur Return), um nach allen Opfern (mindestens ein Bauer) zu suchen.
@echo Gebe 2 ein, um nach Opfern mit mindestens 2 Bauern zu suchen.
@echo Gebe 3 ein, um nach Opfern von Figuren (nicht ausschliesslich Bauern) im Wert von 2 Bauern zu suchen.
set /p input=
set /A whatwork=input
if %whatwork% LSS 1 set /A whatwork=1
if %whatwork% GTR 3 set /A whatwork=3
@echo Gib die maximale Zuglaenge der Gewinnpartien ein, die das Tool untersuchen soll. Fuer Menschpartien
@echo sind mindestens 80 Zuege empfohlen, fuer Enginepartien 90 Zuege. Falls ohne Aufgabefunktion
@echo gespielt wurde, sind mindestens 100 Zuege empfohlen. Fuer eine komplette Suche 250 Zuege eingeben.
@echo off
set /p moves=
set /A movelimit=moves
if %movelimit% LEQ 80 set /A movelimit=80
if %movelimit% GEQ 250 set /A movelimit=250

REM *************************************
REM *** Start of code, do not change! ***
REM *************************************
@echo Lese pgn-file: %gamebase%
set resultbase_nosac=aggressive_no_sacrifices.pgn
set resultbase_withsac=aggressive_with_sacrifices.pgn
set /A resultnumber=0
set /A attackborder=3

del %resultbase_nosac% 2>nul
del %resultbase_withsac% 2>nul

cd ./bin
del results.pgn 2>nul

REM *** Copy the gamebase in a workfile in the bin-folder ***
pgn-extract --quiet ../%gamebase% --output results.pgn >NUL
set /A gesamtanzahl=0

REM *** Count the number of games in the gamebase...
for /f "tokens=2,* delims=PGN: " %%G in ('find /C "[White " results.pgn') do set /A gesamtanzahl=%%H
@echo Zahl der Partien in der pgn-Datei: %gesamtanzahl%

if %gesamtanzahl% LEQ 0 (
  @echo ERROR: No valid games found in the pgn-file
  goto error
)
copy results.pgn temp01.pgn >NUL
del results.pgn 2>nul
REM *** copy the games out of the input-file into a working-file, without draws and ***
REM *** too long games. ***
pgn-extract --quiet -bl20 -bu%movelimit% -Tr1-0 -Tr0-1 temp01.pgn --output results.pgn > NUL
del temp01.pgn 2>nul

REM *** Count the number of remaining (non-draw) games in the gamebase...
for /f "tokens=2,* delims=PGN: " %%G in ('find /C "[White " results.pgn') do set /A anzahl=%%H

del workbase.pgn 2>nul
copy results.pgn workbase.pgn >NUL

@echo Partien zu untersuchen: %anzahl%

REM *** Start the loop for investigating the games one by one ***
set /A counter01=1
:loop01
@echo Partie: %counter01% / %anzahl%

REM *** Read one game out of the gamebase ***
pgn-extract --quiet --selectonly %counter01% workbase.pgn --output onegame.pgn >NUL

REM *** We start after 14 played moves (opening) with move 15 of white ***
set /A cutoffply=28
set /A foundcounter=0

REM *** Cut plies from the beginning of the game ***
pgn-extract --quiet --dropply %cutoffply% onegame.pgn --output workongame.pgn >NUL

REM *** which color won the game, which is investigated? ***
findstr "Result..0-1" onegame.pgn > NUL
if errorlevel 1 (
  set /A side_won=1
) else (
  set /A side_won=0
)

fsutil file createnew sacfound.pgn 0 >NUL

REM *** Check the game, if there is a sacrifice ***
if %side_won% EQU 1 (
  if %whatwork% EQU 1 pgn-extract --quiet -ysacwhite workongame.pgn --output sacfound.pgn >NUL
  if %whatwork% EQU 2 pgn-extract --quiet -ybigsacwhite workongame.pgn --output sacfound.pgn >NUL
  if %whatwork% EQU 3 pgn-extract --quiet -ybigsac_nopawns_white workongame.pgn --output sacfound.pgn >NUL
) else (
  if %whatwork% EQU 1 pgn-extract --quiet -ysacblack workongame.pgn --output sacfound.pgn >NUL
  if %whatwork% EQU 2 pgn-extract --quiet -ybigsacblack workongame.pgn --output sacfound.pgn >NUL
  if %whatwork% EQU 3 pgn-extract --quiet -ybigsac_nopawns_black workongame.pgn --output sacfound.pgn >NUL
)

REM *** Found a sacrifice? Then store it and stop investigating this game ***
find "[White " sacfound.pgn >NUL
if errorlevel 1 (
  REM
) else (
  @echo *** Opfer gefunden ***
  set /A resultnumber=%resultnumber%+1
  pgn-extract --quiet -a../%resultbase_withsac% onegame.pgn >NUL
  goto exitloop03
)

:loop02

fsutil file createnew found.pgn 0 >NUL

REM *** Search for attacking pieces on 6/7/8 rank (white) or 1/2/3 rank (black) ***
REM *** and with enough pieces remaining on the board (see materials-file) ***

if %side_won% EQU 1 (
pgn-extract --quiet --matchplylimit 10 -zmaterials --fenpattern "*[QRBN]*/*/*/*/*/*/*/*" --fenpattern "*/*[QRBN]*/*/*/*/*/*/*" --fenpattern "*/*/*[QRBN]*/*/*/*/*/*" workongame.pgn --output found.pgn >NUL
) else (
pgn-extract --quiet --matchplylimit 10 -zmaterials --fenpattern "*/*/*/*/*/*/*/*[qrbn]*" --fenpattern "*/*/*/*/*/*/*[qrbn]*/*" --fenpattern "*/*/*/*/*/*[qrbn]*/*/*" workongame.pgn --output found.pgn >NUL
)

REM *** Found something? ***
find "[White " found.pgn >NUL
if errorlevel 1 (
  set /A foundpattern=0
) else (
  set /A foundpattern=1
)

if %foundpattern% GTR 0 set /A foundcounter=%foundcounter%+1

REM *** attackborder-value reached? then exit ***
if %foundcounter% GEQ %attackborder% goto exitloop02

REM *** if the move-limit of 60 moves will be reached, before the attackborder-value can be reached, we can exit ***
REM *** this gives a measureable speedup ***
set /A early_exit=(120-((%attackborder%-%foundcounter%)*10))+1
if %cutoffply% GTR %early_exit% goto exitloop02

REM *** move-limit reached? then exit ***
if %cutoffply% GTR 120 goto exitloop02

set /A cutoffply=%cutoffply%+10

REM *** Cut plies from the beginning of the game ***
pgn-extract --quiet --dropply %cutoffply% onegame.pgn --output workongame.pgn >NUL

goto loop02
:exitloop02

REM *** Has the game reached the attackborder-value? ***
REM *** then store this game in the output.pgn file ***
if %foundcounter% GEQ %attackborder% (
  pgn-extract --quiet -a../%resultbase_nosac% onegame.pgn >NUL
  set /A resultnumber=%resultnumber%+1
  @echo *** Figuren auf Reihe 1/2/3 des Gegners ***
) 

:exitloop03
set /A counter01=%counter01%+1

REM *** All games checked? then exit, otherwise go back for next game ***
if %counter01% GTR %anzahl% goto exitloop01
goto loop01
:exitloop01

REM *** delete all working files ***
del results.pgn 2>nul
del whitewins.pgn 2>nul
del blackwins.pgn 2>nul
del blacksacs.pgn 2>nul
del workongame.pgn 2>nul
del found.pgn 2>nul
del onegame.pgn 2>nul
del workbase.pgn 2>nul
del sacfound.pgn 2>nul

cd ../

@echo ****************
@echo *** Fertig ***
@echo ****************
@echo Zahl der Partien in der pgn-Datei: %gesamtanzahl%
@echo %resultnumber% interessante Partien gefunden und gespeichert in:
@echo %resultbase_nosac% UND %resultbase_withsac%

:error
CALL ./bin/playwav %windir%/media/alarm02.wav
pause
